@@ -1,6 +1,19 @@
 #!/usr/bin/env python
 # A simple milter that has grown quite a bit.
 # $Log: bms.py,v $
+# Revision 1.146  2009/08/29 03:38:27  customdesigned
+# Don't ban domains unless gossip score available.
+#
+# Revision 1.145  2009/08/27 21:18:16  customdesigned
+# Track banned domains and expand the offenses that can ban an IP.
+#
+# Revision 1.141  2009/05/20 02:48:18  customdesigned
+# Restrict internal DSNs to official MTAs.
+#
+# Revision 1.140  2009/02/04 02:40:14  customdesigned
+# Parse gossip header before add_spam.  Replace nulls in smtp error txt.
+# Default internal_policy flag false.
+#
 # Revision 1.139  2008/12/22 02:34:39  customdesigned
 # Fix internal policy
 #
@@ -248,6 +261,7 @@ from Milter.greylist import Greylist
 
 from fnmatch import fnmatchcase
 from email.Utils import getaddresses
+from glob import glob
 
 # Import gossip if available
 try:
@@ -322,6 +336,7 @@ block_chinese = False
 case_sensitive_localpart = False
 spam_words = ()
 porn_words = ()
+from_words = ()
 banned_exts = mime.extlist.split(',')
 scan_zip = False
 scan_html = True
@@ -330,6 +345,7 @@ internal_policy = False
 internal_connect = ()
 trusted_relay = ()
 private_relay = ()
+internal_mta = ()
 trusted_forwarder = ()
 internal_domains = ()
 banned_users = ()
@@ -359,6 +375,7 @@ supply_sender = False
 access_file = None
 timeout = 600
 banned_ips = set()
+banned_domains = set()
 greylist = None
 
 logging.basicConfig(
@@ -385,7 +402,8 @@ def read_config(list):
     'supply_sender': 'no',
     'best_guess': 'no',
     'dspam_internal': 'yes',
-    'case_sensitive_localpart': 'no'
+    'case_sensitive_localpart': 'no',
+    'internal_policy': 'no'
   })
   cp.read(list)
   if cp.has_option('milter','datadir'):
@@ -396,7 +414,7 @@ def read_config(list):
   tempfile.tempdir = cp.get('milter','tempdir')
   global socketname, timeout, check_user, log_headers
   global internal_connect, internal_domains, trusted_relay, hello_blacklist
-  global case_sensitive_localpart, private_relay
+  global case_sensitive_localpart, private_relay, internal_mta
   socketname = cp.get('milter','socket')
   timeout = cp.getintdefault('milter','timeout',600)
   check_user = cp.getaddrset('milter','check_user')
@@ -405,12 +423,13 @@ def read_config(list):
   internal_domains = cp.getlist('milter','internal_domains')
   trusted_relay = cp.getlist('milter','trusted_relay')
   private_relay = cp.getlist('milter','private_relay')
+  internal_mta = cp.getlist('milter','internal_mta')
   hello_blacklist = cp.getlist('milter','hello_blacklist')
   case_sensitive_localpart = cp.getboolean('milter','case_sensitive_localpart')
 
   # defang section
   global scan_rfc822, scan_html, block_chinese, scan_zip, block_forward
-  global banned_exts, porn_words, spam_words
+  global banned_exts, porn_words, spam_words, from_words
   if cp.has_section('defang'):
     section = 'defang'
     # for backward compatibility,
@@ -425,6 +444,7 @@ def read_config(list):
   block_forward = cp.getaddrset(section,'block_forward')
   porn_words = cp.getlist(section,'porn_words')
   spam_words = cp.getlist(section,'spam_words')
+  from_words = cp.getlist(section,'from_words')
 
   # scrub section
   global hide_path, reject_virus_from, internal_policy
@@ -655,7 +675,15 @@ cbv_cache = AddrCache(renew=7)
 auto_whitelist = AddrCache(renew=60)
 blacklist = AddrCache(renew=30)
 
-class bmsMilter(Milter.Milter):
+def isbanned(dom,s):
+  if dom in s: return True
+  a = dom.split('.')
+  if a[0] == '*': a = a[1:]
+  if len(a) < 3: return False
+  a[0] = '*'
+  return isbanned('.'.join(a),s)
+
+class bmsMilter(Milter.Base):
   """Milter to replace attachments poisonous to Windows with a WARNING message,
      check SPF, and other anti-forgery features, and implement wiretapping
      and smart alias redirection."""
@@ -739,7 +767,7 @@ class bmsMilter(Milter.Milter):
       self.log("REJECT: spam from self:",hostname)
       self.setreply('550','5.7.1',
         'Your mail server lies.  Its name is *not* %s.' % hostname)
-      return Milter.REJECT
+      return self.offense(inc=4)
     if hostname == 'GC':
       n = gc.collect()
       self.log("gc:",n,' unreachable objects')
@@ -816,9 +844,20 @@ class bmsMilter(Milter.Milter):
     self.discard_list = []
     self.new_headers = []
     self.recipients = []
+    self.confidence = None
     self.cbv_needed = None
     self.whitelist_sender = False
     self.postmaster_reply = False
+    if f == '<>' and internal_mta and self.internal_connection:
+      if not iniplist(self.connectip,internal_mta):
+        self.log("REJECT: pretend MTA at ",self.connectip,
+            " sending MAIL FROM ",f)
+        self.setreply('550','5.7.1',
+        'Your PC is trying to send a DSN even though it is not an MTA.',
+        'If you are running MS Outlook, it is broken.  If you want to',
+        'send return receipts, use a more standards compliant email client.'
+        )
+        return Milter.REJECT
     t = parse_addr(f)
     if len(t) == 2: t[1] = t[1].lower()
     self.canon_from = '@'.join(t)
@@ -858,6 +897,11 @@ class bmsMilter(Milter.Milter):
     self.internal_domain = False
     if len(t) == 2:
       user,domain = t
+      if isbanned(domain,banned_domains):
+        self.log("REJECT: banned domain",domain)
+	self.setreply('550','5.7.1',
+	  '%s has been identified as a spammer domain.')
+	return Milter.REJECT
       for pat in internal_domains:
         if fnmatchcase(domain,pat):
           self.internal_domain = True
@@ -946,7 +990,7 @@ class bmsMilter(Milter.Milter):
       if res not in ('permerror','softfail'):
         self.cbv_needed = None
     elif cbv_cache.has_key(self.canon_from) and cbv_cache[self.canon_from] \
-        or domain in blacklist:
+        or self.canon_from in blacklist:
       # FIXME: don't use cbv_cache for blacklist if policy is 'OK'
       if not self.internal_connection:
         self.offense(inc=2)
@@ -965,7 +1009,7 @@ class bmsMilter(Milter.Milter):
         self.log("BLACKLIST",self.canon_from)
     else:
       # REJECT delayed until after checking whitelist
-      if self.policy == 'REJECT':
+      if self.policy in ('REJECT', 'BAN'):
           self.log('REJECT: no PTR, HELO or SPF')
           self.setreply('550','5.7.1',
     "You must have a valid HELO or publish SPF: http://www.openspf.org ",
@@ -1065,6 +1109,10 @@ class bmsMilter(Milter.Milter):
             "get this bounce, the message was not in fact a forgery, and you",
             "should IMMEDIATELY notify your email administrator of the problem."
           )
+          # Even the most idiotic admin that uses non-existent domains
+          # for helo is not going to forge 'gmail.com'.  So ban the IP too.
+	  if self.hello_name == 'gmail.com':
+	    self.offense(inc=4)
           return Milter.REJECT
         if hres == 'none' and spf_best_guess \
           and not dynip(self.hello_name,self.connectip):
@@ -1097,7 +1145,7 @@ class bmsMilter(Milter.Milter):
         res = 'none'
         policy = p.getNonePolicy()
         if policy in ('CBV','DSN'):
-          self.offenses = 3    # ban ip if any bad recipient
+          self.offense(inc=2)    # ban ip if any bad recipient
         self.need_cbv(policy,q,'strike3')
         # REJECT delayed until after checking whitelist
     if res in ('deny', 'fail'):
@@ -1107,6 +1155,8 @@ class bmsMilter(Milter.Milter):
         # A proper SPF fail error message would read:
         # forger.biz [1.2.3.4] is not allowed to send mail with the domain
         # "forged.org" in the sender address.  Contact <postmaster@forged.org>.
+        if q.d in hello_blacklist:
+          self.offense(inc=4)
         return Milter.REJECT
     elif res == 'softfail':
       if self.need_cbv(p.getSoftfailPolicy(),q,'softfail'):
@@ -1144,7 +1194,7 @@ class bmsMilter(Milter.Milter):
       if self.need_cbv(p.getPermErrorPolicy(),q,'permerror'):
         self.log('REJECT: SPF %s %i %s' % (res,code,txt))
         # latest SPF draft recommends 5.5.2 instead of 5.7.1
-        self.setreply(str(code),'5.5.2',txt,
+        self.setreply(str(code),'5.5.2',txt.replace('\0','^@'),
           'There is a fatal syntax error in the SPF record for %s' % q.o,
           'We cannot accept mail from %s until this is corrected.' % q.o
         )
@@ -1168,16 +1218,18 @@ class bmsMilter(Milter.Milter):
       param = param2dict(str)
       self.notify = param.get('NOTIFY','FAILURE,DELAY').upper().split(',')
       if 'NEVER' in self.notify: self.notify = ()
+      # FIXME: self.notify needs to be by rcpt
       #self.rcpt_param = param
     except:
       self.log("REJECT: invalid PARAM:",to,str)
       self.setreply('550','5.7.1','Invalid RCPT PARAM')
       return Milter.REJECT
     # mail to MAILER-DAEMON is generally spam that bounced
-    if to.startswith('<MAILER-DAEMON@'):
-      self.log('REJECT: RCPT TO:',to,str)
-      self.setreply('550','5.7.1','MAILER-DAEMON does not accept mail')
-      return Milter.REJECT
+    for daemon in ('MAILER-DAEMON','auto-notify'):
+      if to.startswith('<%s@'%daemon):
+        self.log('REJECT: RCPT TO:',to,str)
+        self.setreply('550','5.7.1','%s does not accept mail'%daemon)
+        return Milter.REJECT
     try:
       t = parse_addr(to)
       newaddr = False
@@ -1216,9 +1268,9 @@ class bmsMilter(Milter.Milter):
 	      if auto_whitelist.has_precise_key(self.canon_from):
 		self.log("WHITELIST: DSN from",self.canon_from)
 	      else:
-                if srs_reject_spoofed \
-                  and user.lower() not in ('postmaster','abuse'):
-                  return self.forged_bounce(to)
+                #if srs_reject_spoofed \
+                #  and user.lower() not in ('postmaster','abuse'):
+                #  return self.forged_bounce(to)
                 self.data_allowed = not srs_reject_spoofed
 
         if not self.internal_connection and domain in private_relay:
@@ -1309,15 +1361,16 @@ class bmsMilter(Milter.Milter):
 
       # check for spam that claims to be legal
       lval = val.lower().strip()
-      for adv in ("adv:","adv.","adv ","[adv]","(adv)","advt:","advert:"):
+      for adv in ("adv:","adv.","adv ",
+        "<adv>","<ad>","[adv]","(adv)","advt:","advert:","[spam]"):
         if lval.startswith(adv):
           self.log('REJECT: %s: %s' % (name,val))
-          self.setreply('550','5.7.1','Advertising not accepted here')
+          self.setreply('550','5.7.1','No soliciting allowed')
           return Milter.REJECT
-      for adv in ("adv","(adv)","[adv]"):
+      for adv in ("adv","(adv)","[adv]","(non-spam)"):
         if lval.endswith(adv):
           self.log('REJECT: %s: %s' % (name,val))
-          self.setreply('550','5.7.1','Advertising not accepted here')
+          self.setreply('550','5.7.1','No soliciting allowed')
           return Milter.REJECT
 
       # check for porn keywords
@@ -1343,13 +1396,23 @@ class bmsMilter(Milter.Milter):
 
     elif lname == 'from':
       fname,email = parseaddr(val)
+      for w in spam_words:
+      	if fname.find(w) >= 0:
+          self.log('REJECT: %s: %s' % (name,val))
+          self.setreply('550','5.7.1','No soliciting')
+          return self.bandomain()
+      for w in from_words:
+      	if fname.find(w) >= 0:
+          self.log('REJECT: %s: %s' % (name,val))
+          self.setreply('550','5.7.1','No soliciting')
+          return self.bandomain()
       # check for porn keywords
       lval = fname.lower().strip()
       for w in porn_words:
         if lval.find(w) >= 0:
           self.log('REJECT: %s: %s' % (name,val))
           self.setreply('550','5.7.1','Watch your language')
-          return Milter.REJECT
+          return self.bandomain()
       if email.lower().startswith('postmaster@'):
         # Yes, if From header comes last, this might not help much.
         # But this is a heuristic - if MTAs would send proper DSNs in
@@ -1387,6 +1450,23 @@ class bmsMilter(Milter.Milter):
         "Thank you."
       )
     return Milter.REJECT
+
+  def bandomain(self):
+    if self.spf and self.spf.result == 'pass' and self.confidence == 0:
+      domain = self.canon_from.split('@')[-1]
+      if not isbanned(domain,banned_domains):
+	try:
+	  fp = open('banned_domains','at')
+	  print >>fp,domain 
+	finally: fp.close()
+	banned_domains.add(domain)
+        self.log('BAN DOMAIN:',domain)
+    return Milter.REJECT
+
+  def data(self):
+    if not self.data_allowed:
+      return self.forged_bounce()
+    return Milter.CONTINUE
     
   def header(self,name,hval):
     if not self.data_allowed:
@@ -1484,12 +1564,20 @@ class bmsMilter(Milter.Milter):
         self.add_header('X-DSpam-HeaderScore','%f'%ds.probability)
       finally:
         ds.destroy()
+    self.ioerr = None
     return Milter.CONTINUE
 
+  @Milter.noreply
   def body(self,chunk):         # copy body to temp file
-    if self.fp:
-      self.fp.write(chunk)      # IOError causes TEMPFAIL in milter
-      self.bodysize += len(chunk)
+    try:
+      if self.fp:
+        self.fp.write(chunk)      # IOError causes TEMPFAIL in milter
+        self.bodysize += len(chunk)
+    except Exception,x:
+      if not self.ioerr:
+        self.ioerr = x
+	self.log(x)
+      self.fp = None
     return Milter.CONTINUE
 
   def _headerChange(self,msg,name,value):
@@ -1571,11 +1659,11 @@ class bmsMilter(Milter.Milter):
               sender = dspam_users.get(self.canon_from)
               if sender:
                 self.log("SPAM: %s" % sender)   # log user for SPAM
-                ds.add_spam(sender,txt)
-                txt = None
                 self.fp.seek(0)
                 self.gossip_header()
                 self.fp = None
+                ds.add_spam(sender,txt)
+                txt = None
                 return Milter.DISCARD
             elif user == 'falsepositive' and self.internal_connection:
               sender = dspam_users.get(self.canon_from)
@@ -1747,10 +1835,19 @@ class bmsMilter(Milter.Milter):
       if self.mailfrom != '<>' and not self.cbv_needed:
         self.cbv_needed = (q,tname)
     elif policy != 'OK':
+      if policy == 'BAN':
+	self.offense(inc=3)
+      elif self.offenses:
+        self.offense()	 # multiple forged domains are extra evil
       return True
     return False
 
   def eom(self):
+    if self.ioerr:
+      fname = tempfile.mktemp(".ioerr")  # save message that caused crash
+      os.rename(self.tempname,fname)
+      self.tempname = None
+      return Milter.TEMPFAIL
     if not self.fp:
       return Milter.ACCEPT      # no message collected - so no eom processing
 
@@ -2013,7 +2110,6 @@ def main():
       milter_log.error('Unable to read: %s',access_file)
       return
   try:
-    from glob import glob
     global banned_ips
     banned_ips = set(addr2bin(ip) 
         for fn in glob('banned_ips*')
@@ -2021,6 +2117,16 @@ def main():
     print len(banned_ips),'banned ips'
   except:
     milter_log.exception('Error reading banned_ips')
+
+  try:
+    global banned_domains
+    banned_domains = set(dom.strip()
+	    for fn in glob('banned_domains*')
+	    for dom in open(fn))
+    print len(banned_domains),'banned domains'
+  except:
+    milter_log.exception('Error reading banned_domains')
+
   Milter.factory = bmsMilter
   flags = Milter.CHGBODY + Milter.CHGHDRS + Milter.ADDHDRS
   if wiretap_dest or smart_alias or dspam_userdir:
