@@ -10,13 +10,14 @@
 <li><a href="logmsgs.html">Log&nbsp;Messages</a>
 <li><a href="http://bmsi.com/mailman/listinfo/pymilter">Mailing&nbsp;List</a>
 <li><a href="credits.html">CREDITS</a>
-<li><a href="http://sourceforge.net"><img src="http://sflogo.sourceforge.net/sflogo.php?group_id=139894&amp;type=1" width="88" height="31" border="0" alt="SourceForge.net Logo" /></a>
+<li><a href="http://sourceforge.net"><img src="sflogo.png" width="88" height="31" border="0" alt="SourceForge.net Logo"></a>
 <h3>Links</h3>
 <li><a href="https://www.milter.org/developers/api/index">C&nbsp;API</a>
 <li><a href="http://www.milter.org/">Milter.Org</a>
 <li><a href="http://www.python.org/">Python.Org</a>
 <li><a href="http://www.sendmail.org/">Sendmail.Org</a>
 <li><a href="http://www.openspf.org/">SPF</a>
+<li><a href="http://spidey2.bmsi.com/pymilter/">pymilter</a>
 <li><a href="pysrs.html">pysrs</a>
 <li><a href="http://cheeseshop.python.org/pypi/pyspf">pyspf</a>
 <li><a href="http://bmsi.com/python/pygossip.html">pygossip</a>
